/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) 2016 OX Software GmbH
 * Mail: info@open-xchange.com
 *
 * @author Daniel Rentz <daniel.rentz@open-xchange.com>
 */

define([
    'io.ox/office/spreadsheet/view/labels'
], function (Labels) {

    'use strict';

    // static class Labels ====================================================

    describe('Spreadsheet module Labels', function () {

        it('should exist', function () {
            expect(Labels).to.be.an('object');
        });

        // static methods -----------------------------------------------------

        describe('method "getColLabel"', function () {
            it('should exist', function () {
                expect(Labels).itself.to.respondTo('getColLabel');
            });
            it('should return the English column label', function () {
                expect(Labels.getColLabel(0)).to.equal('Column A');
                expect(Labels.getColLabel(1)).to.equal('Column B');
                expect(Labels.getColLabel(25)).to.equal('Column Z');
                expect(Labels.getColLabel(26)).to.equal('Column AA');
            });
        });

        describe('method "getRowLabel"', function () {
            it('should exist', function () {
                expect(Labels).itself.to.respondTo('getRowLabel');
            });
            it('should return the English row label', function () {
                expect(Labels.getRowLabel(0)).to.equal('Row 1');
                expect(Labels.getRowLabel(1)).to.equal('Row 2');
                expect(Labels.getRowLabel(8)).to.equal('Row 9');
                expect(Labels.getRowLabel(9)).to.equal('Row 10');
            });
        });

        describe('method "getColRowLabel"', function () {
            it('should exist', function () {
                expect(Labels).itself.to.respondTo('getColRowLabel');
            });
            it('should return the English column label', function () {
                expect(Labels.getColRowLabel(0, true)).to.equal('Column A');
                expect(Labels.getColRowLabel(1, true)).to.equal('Column B');
                expect(Labels.getColRowLabel(25, true)).to.equal('Column Z');
                expect(Labels.getColRowLabel(26, true)).to.equal('Column AA');
            });
            it('should return the English row label', function () {
                expect(Labels.getColRowLabel(0, false)).to.equal('Row 1');
                expect(Labels.getColRowLabel(1, false)).to.equal('Row 2');
                expect(Labels.getColRowLabel(8, false)).to.equal('Row 9');
                expect(Labels.getColRowLabel(9, false)).to.equal('Row 10');
            });
        });

        describe('method "getCategoryLabel"', function () {
            it('should exist', function () {
                expect(Labels).itself.to.respondTo('getCategoryLabel');
            });
            it('should return the English category label', function () {
                expect(Labels.getCategoryLabel('math')).to.equal('Math');
            });
        });
    });

    // ========================================================================
});
